@@ -3,14 +3,21 @@ import os, sys, shutil, errno, time, signal
 from StringIO import StringIO
 from twisted.python import usage
 from twisted.internet import defer
+from twisted.scripts import twistd
 
 # does "flappserver start" need us to refrain from importing the reactor here?
+# A: probably, to allow --reactor= to work
 import foolscap
 from foolscap.api import Tub, Referenceable, fireEventually
 from foolscap.pb import generateSwissnumber
 from foolscap.appserver.services import build_service, BadServiceArguments
 from foolscap.appserver.server import AppServer
 
+def get_umask():
+    oldmask = os.umask(0)
+    os.umask(oldmask)
+    return oldmask
+
 class CreateOptions(usage.Options):
     synopsis = "Usage: flappserver create [options] BASEDIR"
 
@@ -20,14 +27,20 @@ class CreateOptions(usage.Options):
     optParameters = [
         ("port", "p", "tcp:0", "TCP port to listen on (strports string)"),
         ("location", "l", "", "Tub location hints to use in generated FURLs. An empty location means to generate one automatically, by looking at the active network interfaces."),
+        ("umask", None, None, "(octal) file creation mask to use for the server. If not provided, the current umask (%04o) is copied." % get_umask()),
         ]
 
     def opt_port(self, port):
         assert not port.startswith("ssl:")
         self["port"] = port
+    def opt_umask(self, value):
+        self["umask"] = int(value, 8)
 
     def parseArgs(self, basedir):
         self.basedir = basedir
+    def postOptions(self):
+        if self["umask"] is None:
+            self["umask"] = get_umask()
 
 FLAPPSERVER_TACFILE = """\
 # -*- python -*-
@@ -62,6 +75,7 @@ class Create:
 
         os.makedirs(basedir)
         os.makedirs(os.path.join(basedir, "services"))
+        os.chmod(basedir, 0700)
 
         # start the server and let it run briefly. This lets the Tub spin up,
         # create the key, decide upon a port, and auto-determine its location
@@ -79,6 +93,10 @@ class Create:
         f.write("%s\n" % options["location"])
         f.close()
 
+        f = open(os.path.join(basedir, "umask"), "w")
+        f.write("%04o\n" % options["umask"])
+        f.close()
+
         self.server = None
         d = fireEventually(basedir)
         d.addCallback(AppServer, stdout)
@@ -248,19 +266,6 @@ class StartOptions(usage.Options):
         self.basedir = basedir
         self.twistd_args = twistd_args
 
-def try_to_run_command(command, args):
-    # if it works, this will not return
-    # if the command is not found (ENOENT), this returns False
-    # some other exception might be raised
-    argv = [command] + list(args)
-    try:
-        os.execvp(command, argv)
-        # doesn't return
-    except OSError, e:
-        if e.errno == errno.ENOENT:
-            return False
-        raise
-
 class Start:
     def run(self, options):
         basedir = options.basedir
@@ -273,34 +278,11 @@ class Start:
             print >>stderr, "%s does not look like a node directory (no .tac file)" % basedir
             return 1
 
-        # this requires that "twistd" or "twistd.exe" be somewhere on your
-        # $PATH. It will probably fail on windows if you must run "python
-        # twistd.py" or something like that. I'm not sure it's appropriate to
-        # try to accomodate argv[0]="python": too many decisions to make
-        # about how to run some tool which ought to have a proper shbang line
-        # of its own. It will certainly fail if you don't have Twisted and
-        # twistd installed somewhere: I don't think it's appropriate to try
-        # to run a twistd that was installed outside of $PATH.
-
-        args = ["--no_save", "--python", tac] + list(options.twistd_args)
-
-        print >>stderr, "Launching Server..."
         os.chdir(options.basedir)
-        try_to_run_command("twistd", args)
-        # if we get here, we couldn't find twistd
-
-        if sys.platform == "win32":
-            try_to_run_command("twistd.exe", args)
-            # if we get here, we couldn't find twistd.exe
-
-            print >>stderr, "twistd.exe: command not found"
-            print >>stderr, "Neither 'twistd' nor 'twistd.exe' were found on $PATH"
-        else:
-            print >>stderr, "twistd: command not found"
-            print >>stderr, "'twistd' was not found on $PATH"
-
-        print >>stderr, "You must install Twisted (and its bin/twistd) to use this command"
-        return 127
+        twistd_args = list(options.twistd_args)
+        sys.argv[1:] = ["--no_save", "--python", tac] + twistd_args
+        print >>stderr, "Launching Server..."
+        twistd.run()
 
 
 class StopOptions(usage.Options):
