@@ -1,5 +1,5 @@
 
-from twisted.python import failure, log, reflect
+from twisted.python import failure, reflect
 from twisted.internet import defer
 
 from foolscap import copyable, slicer, tokens
@@ -8,7 +8,10 @@ from foolscap.constraint import ByteStringConstraint
 from foolscap.slicers.list import ListConstraint
 from tokens import BananaError, Violation
 from foolscap.util import AsyncAND
+from foolscap.logging import log
 
+def wrap_remote_failure(f):
+    return failure.Failure(tokens.RemoteException(f))
 
 class FailureConstraint(AttributeDictConstraint):
     opentypes = [("copyable", "twisted.python.failure.Failure")]
@@ -32,18 +35,23 @@ class PendingRequest(object):
     # this object is a local representation of a message we have sent to
     # someone else, that will be executed on their end.
     active = True
-    methodName = None # for debugging
 
-    def __init__(self, reqID, rref=None):
+    def __init__(self, reqID, rref, interface_name, method_name):
         self.reqID = reqID
         self.rref = rref # keep it alive
         self.broker = None # if set, the broker knows about us
         self.deferred = defer.Deferred()
         self.constraint = None # this constrains the results
+        self.failure = None
+        self.interface_name = interface_name # for error messages
+        self.method_name = method_name # same
 
     def setConstraint(self, constraint):
         self.constraint = constraint
 
+    def getMethodNameInfo(self):
+        return (self.interface_name, self.method_name)
+
     def complete(self, res):
         if self.broker:
             self.broker.removeRequest(self)
@@ -62,22 +70,36 @@ class PendingRequest(object):
             if (self.broker and
                 self.broker.tub and
                 self.broker.tub.logRemoteFailures):
-                log.msg("an outbound callRemote (that we sent to someone "
-                        "else) failed on the far end")
+
+                my_short_tubid = "??"
+                if self.broker.tub: # for tests
+                    my_short_tubid = self.broker.tub.getShortTubID()
+                their_short_tubid = self.broker.remote_tubref.getShortTubID()
+
+                lp = log.msg("an outbound callRemote (that we [%s] sent to "
+                             "someone else [%s]) failed on the far end"
+                             % (my_short_tubid, their_short_tubid),
+                             level=log.UNUSUAL)
+                methname = ".".join([self.interfaceName or "?",
+                                     self.methodName or "?"])
                 log.msg(" reqID=%d, rref=%s, methname=%s"
-                        % (self.reqID, self.rref, self.methodName))
-                stack = why.getTraceback()
+                        % (self.reqID, self.rref, methname),
+                        level=log.NOISY, parent=lp)
+                #stack = why.getTraceback()
                 # TODO: include the first few letters of the remote tubID in
                 # this REMOTE tag
-                stack = "REMOTE: " + stack.replace("\n", "\nREMOTE: ")
-                log.msg(" the failure was:")
-                log.msg(stack)
+                #stack = "REMOTE: " + stack.replace("\n", "\nREMOTE: ")
+                log.msg(" the REMOTE failure was:", failure=why,
+                        level=log.NOISY, parent=lp)
+                #log.msg(stack, level=log.NOISY, parent=lp)
             self.deferred.errback(why)
         else:
-            log.msg("multiple failures")
-            log.msg("first one was:", self.failure)
-            log.msg("this one was:", why)
-            log.err("multiple failures indicate a problem")
+            log.msg("WEIRD: fail() on an inactive request", traceback=True)
+            if self.failure:
+                log.msg("multiple failures")
+                log.msg("first one was:", self.failure)
+                log.msg("this one was:", why)
+                log.err("multiple failures indicate a problem")
 
 class ArgumentSlicer(slicer.ScopedSlicer):
     opentype = ('arguments',)
@@ -124,7 +146,7 @@ class CallSlicer(slicer.ScopedSlicer):
     def describe(self):
         return "<call-%s-%s-%s>" % (self.reqID, self.clid, self.methodname)
 
-class InboundDelivery:
+class InboundDelivery(object):
     """An inbound message that has not yet been delivered.
 
     This is created when a 'call' sequence has finished being received. The
@@ -153,9 +175,10 @@ class InboundDelivery:
     above any cycles.
     """
 
-    def __init__(self, reqID, obj,
+    def __init__(self, broker, reqID, obj,
                  interface, methodname, methodSchema,
                  allargs):
+        self.broker = broker
         self.reqID = reqID
         self.obj = obj
         self.interface = interface
@@ -165,20 +188,35 @@ class InboundDelivery:
 
     def logFailure(self, f):
         # called if tub.logLocalFailures is True
-        log.msg("an inbound callRemote that we executed (on behalf of "
-                "someone else) failed")
-        log.msg(" reqID=%d, rref=%s, methname=%s" %
-                (self.reqID, self.obj, self.methodname))
-        log.msg(" args=%s" % (self.allargs.args,))
-        log.msg(" kwargs=%s" % (self.allargs.kwargs,))
-        if isinstance(f.type, str):
-            stack = "getTraceback() not available for string exceptions\n"
+        my_short_tubid = "??"
+        if self.broker.tub: # for tests
+            my_short_tubid = self.broker.tub.getShortTubID()
+        their_short_tubid = "<unauth>"
+        if self.broker.remote_tubref:
+            their_short_tubid = self.broker.remote_tubref.getShortTubID()
+        lp = log.msg("an inbound callRemote that we [%s] executed (on behalf "
+                     "of someone else, TubID %s) failed"
+                     % (my_short_tubid, their_short_tubid),
+                     level=log.UNUSUAL)
+        if self.interface:
+            methname = self.interface.getName() + "." + self.methodname
         else:
-            stack = f.getTraceback()
+            methname = self.methodname
+        log.msg(" reqID=%d, rref=%s, methname=%s" %
+                (self.reqID, self.obj, methname),
+                level=log.NOISY, parent=lp)
+        log.msg(" args=%s" % (self.allargs.args,), level=log.NOISY, parent=lp)
+        log.msg(" kwargs=%s" % (self.allargs.kwargs,),
+                level=log.NOISY, parent=lp)
+        #if isinstance(f.type, str):
+        #    stack = "getTraceback() not available for string exceptions\n"
+        #else:
+        #    stack = f.getTraceback()
         # TODO: trim stack to everything below Broker._doCall
-        stack = "LOCAL: " + stack.replace("\n", "\nLOCAL: ")
-        log.msg(" the failure was:")
-        log.msg(stack)
+        #stack = "LOCAL: " + stack.replace("\n", "\nLOCAL: ")
+        log.msg(" the LOCAL failure was:", failure=f,
+                level=log.NOISY, parent=lp)
+        #log.msg(stack, level=log.NOISY, parent=lp)
 
 class ArgumentUnslicer(slicer.ScopedUnslicer):
     methodSchema = None
@@ -447,7 +485,7 @@ class CallUnslicer(slicer.ScopedUnslicer):
             # exception for unknown methods
 
             # must find the schema, using the interfaces
-            
+
             # TODO: getSchema should probably be in an adapter instead of in
             # a pb.Referenceable base class. Old-style (unconstrained)
             # flavors.Referenceable should be adapted to something which
@@ -502,7 +540,7 @@ class CallUnslicer(slicer.ScopedUnslicer):
         if self.stage != 4:
             raise BananaError("'call' sequence ended too early")
         # time to create the InboundDelivery object so we can queue it
-        delivery = InboundDelivery(self.reqID, self.obj,
+        delivery = InboundDelivery(self.broker, self.reqID, self.obj,
                                    self.interface, self.methodname,
                                    self.methodSchema,
                                    self.allargs)
@@ -608,7 +646,7 @@ class AnswerUnslicer(slicer.ScopedUnslicer):
         # if the Violation was received after we got the reqID, we can tell
         # the broker it was an error
         if self.request != None:
-            self.request.fail(f)
+            self.request.fail(f) # local violation
         return f # give up our sequence
 
     def receiveClose(self):
@@ -635,6 +673,10 @@ class AnswerUnslicer(slicer.ScopedUnslicer):
         def _done(res):
             self.request.complete(res)
         def _fail(f):
+            # we hit here if any of the _ready_deferreds fail (i.e a Gift
+            # failed to resolve), or if the _child_deferred fails (not sure
+            # how this could happen). I think it's ok to return a local
+            # exception (instead of a RemoteException) for both.
             self.request.fail(f)
         d.addCallbacks(_done, _fail)
 
@@ -702,7 +744,10 @@ class ErrorUnslicer(slicer.ScopedUnslicer):
             self.gotFailure = True
 
     def receiveClose(self):
-        self.request.fail(self.failure)
+        f = self.failure
+        if not self.broker._expose_remote_exception_types:
+            f = wrap_remote_failure(f)
+        self.request.fail(f)
         return None, None
 
     def describe(self):
@@ -711,6 +756,12 @@ class ErrorUnslicer(slicer.ScopedUnslicer):
         return "<error-%s>" % self.request.reqID
 
 
+def truncate(s, limit):
+    assert limit > 3
+    if s and len(s) > limit:
+        s = s[:limit-3] + ".."
+    return s
+
 # failures are sent as Copyables
 class FailureSlicer(slicer.BaseSlicer):
     slices = failure.Failure
@@ -726,7 +777,7 @@ class FailureSlicer(slicer.BaseSlicer):
             yield v
     def describe(self):
         return "<%s>" % self.classname
-        
+
     def getStateToCopy(self, obj, broker):
         #state = obj.__dict__.copy()
         #state['tb'] = None
@@ -749,6 +800,12 @@ class FailureSlicer(slicer.BaseSlicer):
         else:
             state['value'] = str(obj.value) # Exception instance
             state['type'] = reflect.qual(obj.type) # Exception class
+        # TODO: I suspect that f.value may be getting a copy of the
+        # traceback, because I've seen it be 1819 bytes at one point. I had
+        # assumed that it was just the exception name plus args: whatever
+        # Exception.__repr__ returns.
+        state['value'] = truncate(state['value'], 1000)
+        state['type'] = truncate(state['type'], 200)
 
         if broker.unsafeTracebacks:
             if isinstance(obj.type, str):
@@ -769,7 +826,12 @@ class FailureSlicer(slicer.BaseSlicer):
                                   "\n\n-- TRACEBACK ELIDED --\n\n"
                                   + state['traceback'][-1200:])
 
-        state['parents'] = obj.parents
+        parents = obj.parents[:]
+        if parents:
+            for i,value in enumerate(parents):
+                parents[i] = truncate(value, 200)
+        state['parents'] = parents
+
         return state
 
 class CopiedFailure(failure.Failure, copyable.RemoteCopyOldStyle):
@@ -797,6 +859,18 @@ class CopiedFailure(failure.Failure, copyable.RemoteCopyOldStyle):
     def __init__(self):
         copyable.RemoteCopyOldStyle.__init__(self)
 
+    def __getstate__(self):
+        s = failure.Failure.__getstate__(self)
+        # the ExceptionLikeString we use in self.type is not pickleable, so
+        # replace it with the same sort of string that we use in the wire
+        # protocol.
+        if not isinstance(self.type, str):
+            s['type'] = reflect.qual(self.type)
+        return s
+
+    def __setstate__(self, state):
+        self.setCopyableState(state)
+
     def setCopyableState(self, state):
         #self.__dict__.update(state)
         self.__dict__ = state
@@ -848,7 +922,7 @@ copyable.registerRemoteCopy(FailureSlicer.classname, CopiedFailure)
 class CopiedFailureSlicer(FailureSlicer):
     # A calls B. B calls C. C fails and sends a Failure to B. B gets a
     # CopiedFailure and sends it to A. A should get a CopiedFailure too. This
-    # class lives on B and slicers the CopiedFailure as it is sent to A.
+    # class lives on B and slices the CopiedFailure as it is sent to A.
     slices = CopiedFailure
 
     def getStateToCopy(self, obj, broker):
