@@ -499,7 +499,7 @@ class Tub(service.MultiService):
         This method returns a Deferred that will fire once the location is
         actually established. Calls to registerReference() must be put off
         until the location has been set. And of course, you must call
-        listenOn() before calling autoSetLocation()."""
+        listenOn() before calling setLocationAutomatically()."""
 
         # first, make sure the reactor is actually running, by using the
         # eventual-send queue
@@ -738,7 +738,10 @@ class Tub(service.MultiService):
                 if ref not in self.referenceToName:
                     self.referenceToName[ref] = name
                 return ref
-        raise KeyError("unable to find reference for name '%s'" % (name,))
+        # don't reveal the full swissnum
+        hint = name[:2]
+        raise KeyError("unable to find reference for name starting with '%s'"
+                       % hint)
 
     def getReferenceForURL(self, url):
         # TODO: who should this be used by?
@@ -1076,12 +1079,3 @@ class UnauthenticatedTub(Tub):
         return "<unauth>"
     def getShortTubID(self):
         return "<unauth>"
-
-def getRemoteURL_TCP(host, port, pathname, *interfaces):
-    url = "pb://%s:%d/%s" % (host, port, pathname)
-    if crypto_available:
-        s = Tub()
-    else:
-        s = UnauthenticatedTub()
-    d = s.getReference(url, interfaces)
-    return d
