@@ -19,9 +19,9 @@ from foolscap.eventual import flushEventualQueue
 from foolscap.test.common import HelperTarget, TargetMixin, ShouldFailMixin
 from foolscap.test.common import RIMyTarget, Target, TargetWithoutInterfaces, \
      BrokenTarget
-from foolscap import DeadReferenceError
-from foolscap.api import RemoteException, UnauthenticatedTub
+from foolscap.api import RemoteException, UnauthenticatedTub, DeadReferenceError
 from foolscap.call import CopiedFailure
+from foolscap.logging import log as flog
 
 class Unsendable:
     pass
@@ -93,6 +93,8 @@ class TestCall(TargetMixin, ShouldFailMixin, unittest.TestCase):
         # f should be a CopiedFailure
         self.failUnless(isinstance(f, Failure),
                         "Hey, we didn't fail: %s" % f)
+        self.failUnless(isinstance(f, CopiedFailure),
+                        "not CopiedFailure: %s" % f)
         self.failUnless(f.check(ValueError),
                         "wrong exception type: %s" % f)
         self.failUnlessSubstring("you asked me to fail", f.value)
@@ -408,6 +410,13 @@ class TestCall(TargetMixin, ShouldFailMixin, unittest.TestCase):
         d = self.shouldFail(DeadReferenceError, "lost_is_deadref.1",
                             "Connection was lost",
                             get_d)
+        def _examine_error((f,)):
+            # the (to tubid=XXX) part will see "tub=call", which is an
+            # abbreviation of "callingBroker" as created in
+            # TargetMixin.setupBrokers
+            self.failUnlessIn("(to tubid=call)", str(f.value))
+            self.failUnlessIn("(during method=None:hang)", str(f.value))
+        d.addCallback(_examine_error)
         # and once the connection is down, we should get a DeadReferenceError
         # for new messages
         d.addCallback(lambda res:
@@ -431,10 +440,12 @@ class TestCall(TargetMixin, ShouldFailMixin, unittest.TestCase):
     def testNotifyOnDisconnect(self):
         rr, target = self.setupTarget(HelperTarget())
         self.lost = 0
+        self.failUnlessEqual(rr.isConnected(), True)
         rr.notifyOnDisconnect(self.disconnected)
         rr.tracker.broker.transport.loseConnection(Failure(CONNECTION_LOST))
         d = flushEventualQueue()
         def _check(res):
+            self.failUnlessEqual(rr.isConnected(), False)
             self.failUnless(self.lost)
             self.failUnlessEqual(self.lost_args, ((),{}))
             # it should be safe to unregister now, even though the callback
@@ -532,6 +543,9 @@ class ExamineFailuresMixin:
         self.failUnless(isinstance(f2, CopiedFailure))
         self.failUnlessSubstring("you asked me to fail", f2.value)
         self.failIf(f2.check(RemoteException))
+        l = flog.FoolscapLogger()
+        l.msg("f1", failure=f)
+        l.msg("f2", failure=f2)
 
     def _examine_local_violation(self, r):
         f = r[0]
@@ -801,10 +815,6 @@ class ReferenceCounting(ShouldFailMixin, unittest.TestCase):
             # forget
             rref.tracker.interfaceName = None
             rref.tracker.interface = None
-            for b in self.target_tub.unauthenticatedBrokers:
-                b.debugReceive = True
-                #print b
-                pass
             self.rref = rref
         d.addCallback(_stash)
 
@@ -834,7 +844,6 @@ class ReferenceCounting(ShouldFailMixin, unittest.TestCase):
         def _check_shared(ign):
             # the off-by-two bug would cause the second tuple shared-ref to
             # point at the set instead of the first tuple
-            print target.obj
             self.failUnlessEqual(type(target.obj), list)
             one, two, three, four = target.obj
             self.failUnlessEqual(type(one), tuple)
