@@ -12,9 +12,9 @@ from foolscap.logging import gatherer, log, tail, incident, cli, web, \
      publish, dumper
 from foolscap.logging.interfaces import RILogObserver
 from foolscap.eventual import fireEventually, flushEventualQueue
-from foolscap import Referenceable
 from foolscap.tokens import NoLocationError
 from foolscap.test.common import PollMixin, StallMixin, GoodEnoughTub
+from foolscap.api import RemoteException, Referenceable
 
 
 class Basic(unittest.TestCase):
@@ -43,6 +43,18 @@ class Basic(unittest.TestCase):
         l = log.FoolscapLogger()
         l.msg("how did we get here?", stacktrace=True)
 
+    def testFailure(self):
+        l = log.FoolscapLogger()
+        f1 = failure.Failure(ValueError("bad value"))
+        l.msg("failure1", failure=f1)
+        # real RemoteExceptions always wrap CopiedFailure, so this is not
+        # really accurate. However, it's a nuisance to create a real
+        # CopiedFailure: look in
+        # test_call.ExamineFailuresMixin._examine_raise for test code that
+        # exercises this properly.
+        f2 = failure.Failure(RemoteException(f1))
+        l.msg("failure2", failure=f2)
+
     def testParent(self):
         l = log.FoolscapLogger()
         p1 = l.msg("operation requested", level=log.OPERATIONAL)
@@ -356,7 +368,7 @@ class Incidents(unittest.TestCase, PollMixin, LogfileReaderMixin):
         l.setIncidentReporterFactory(incident.NonTrailingIncidentReporter)
         l.setLogDir("logging/Incidents/classify")
         got_logdir = l.logdir
-        l.msg("foom", level=log.BAD)
+        l.msg("foom", level=log.BAD, failure=failure.Failure(RuntimeError()))
         d = fireEventually()
         def _check(res):
             files = [fn for fn in os.listdir(got_logdir) if fn.endswith(".bz2")]
@@ -373,6 +385,20 @@ class Incidents(unittest.TestCase, PollMixin, LogfileReaderMixin):
             ic.run(options)
             out = options.stdout.getvalue()
             self.failUnless(out.strip().endswith(": foom"), out)
+
+            ic2 = incident.IncidentClassifier()
+            options = incident.ClassifyOptions()
+            options.parseOptions(["--verbose"] +
+                                 [os.path.join(got_logdir, fn) for fn in files])
+            options.stdout = StringIO()
+            ic2.run(options)
+            out = options.stdout.getvalue()
+            self.failUnless(".flog.bz2: unknown\n" in out, out)
+            # this should have a pprinted trigger dictionary
+            self.failUnless("'message': 'foom'," in out, out)
+            self.failUnless("'num': 0," in out, out)
+            self.failUnless("RuntimeError" in out, out)
+
         d.addCallback(_check)
         return d
 
@@ -672,10 +698,11 @@ class Publish(PollMixin, unittest.TestCase):
                 ob.last_received = time.time()
                 log.msg("this is a later message")
             d.addCallback(_emit)
-            # wait until the observer has seen nothing for a full second
+            # wait until we've received the later message
             def _check_f():
-                if ob.last_received < time.time() - 1.0:
-                    return True
+                for m in ob.messages:
+                    if m.get("message") == "this is a later message":
+                        return True
                 return False
             d.addCallback(lambda res: self.poll(_check_f))
             def _check_observer(res):
@@ -1404,6 +1431,17 @@ class Gatherer(unittest.TestCase, LogfileReaderMixin, StallMixin, PollMixin):
         gatherer2_furl = gatherer2.my_furl
         starting_timestamp2 = gatherer2._starting_timestamp
 
+        gatherer3_basedir = os.path.join(basedir, "gatherer3")
+        os.makedirs(gatherer3_basedir)
+        gatherer3 = MyGatherer(None, False, gatherer3_basedir)
+        gatherer3.tub_class = GoodEnoughTub
+        gatherer3.d = defer.Deferred()
+        gatherer3.setServiceParent(self.parent)
+        # that will start the gatherer
+        fn3 = gatherer3._savefile_name
+        gatherer3_furl = gatherer3.my_furl
+        starting_timestamp3 = gatherer3._starting_timestamp
+
         gatherer_furlfile = os.path.join(basedir, "log_gatherer.furl")
         f = open(gatherer_furlfile, "w")
         f.write(gatherer1_furl + "\n")
@@ -1414,19 +1452,22 @@ class Gatherer(unittest.TestCase, LogfileReaderMixin, StallMixin, PollMixin):
         expected_tubid = t.tubID
         if t.tubID is None:
             expected_tubid = "<unauth>"
+        t.setOption("log-gatherer-furl", gatherer3_furl)
         t.setServiceParent(self.parent)
         l = t.listenOn("tcp:0:interface=127.0.0.1")
         t.setLocation("127.0.0.1:%d" % l.getPortnum())
         t.setOption("log-gatherer-furlfile", gatherer_furlfile)
         # now both log gatherer connections will be being established
 
-        d = defer.DeferredList([gatherer1.d, gatherer2.d],
+        d = defer.DeferredList([gatherer1.d, gatherer2.d, gatherer3.d],
                                fireOnOneErrback=True)
         d.addCallback(self._emit_messages_and_flush, t)
         d.addCallback(lambda res: gatherer1.do_rotate())
         d.addCallback(self._check_gatherer, starting_timestamp1, expected_tubid)
         d.addCallback(lambda res: gatherer2.do_rotate())
         d.addCallback(self._check_gatherer, starting_timestamp2, expected_tubid)
+        d.addCallback(lambda res: gatherer3.do_rotate())
+        d.addCallback(self._check_gatherer, starting_timestamp3, expected_tubid)
         return d
     test_log_gatherer_furlfile_multiple.timeout = 20
 
@@ -1713,12 +1754,21 @@ class Dumper(unittest.TestCase, LogfileWriterMixin, LogfileReaderMixin):
         d = self.create_logfile()
         def _check(fn):
             events = self._read_logfile(fn)
+
             d = dumper.LogDumper()
+            # initialize the LogDumper() timestamp mode
+            d.options = dumper.DumpOptions()
+            d.options.parseOptions([fn])
 
             argv = ["flogtool", "dump", fn]
             (out,err) = cli.run_flogtool(argv[1:], run_by_human=False)
             self.failUnlessEqual(err, "")
             lines = list(StringIO(out).readlines())
+            self.failUnless(lines[0].strip().startswith("Application versions"),
+                            lines[0])
+            mypid = os.getpid()
+            self.failUnlessEqual(lines[3].strip(), "PID: %s" % mypid, lines[3])
+            lines = lines[5:]
             line0 = "local#%d %s: one" % (events[1]["d"]["num"],
                                           d.format_time(events[1]["d"]["time"]))
             self.failUnlessEqual(lines[0].strip(), line0)
@@ -1742,6 +1792,11 @@ class Dumper(unittest.TestCase, LogfileWriterMixin, LogfileReaderMixin):
             (out,err) = cli.run_flogtool(argv[1:], run_by_human=False)
             self.failUnlessEqual(err, "")
             lines = list(StringIO(out).readlines())
+            self.failUnless(lines[0].strip().startswith("Application versions"),
+                            lines[0])
+            mypid = os.getpid()
+            self.failUnlessEqual(lines[3].strip(), "PID: %s" % mypid, lines[3])
+            lines = lines[5:]
             line0 = "local#%d rx(%s) emit(%s): one" % \
                     (events[1]["d"]["num"],
                      d.format_time(events[1]["rx_time"]),
@@ -1774,11 +1829,17 @@ class Dumper(unittest.TestCase, LogfileWriterMixin, LogfileReaderMixin):
             (out,err) = cli.run_flogtool(argv[1:], run_by_human=False)
             self.failUnlessEqual(err, "")
             lines = list(StringIO(out).readlines())
-            self.failUnlessEqual(len(lines), 3)
-            self.failIf("[INCIDENT-TRIGGER]" in lines[0])
-            self.failIf("[INCIDENT-TRIGGER]" in lines[1])
-            self.failUnless(lines[2].strip().endswith(": boom [INCIDENT-TRIGGER]"))
-
+            self.failUnlessEqual(len(lines), 8)
+            self.failUnlessEqual(lines[0].strip(),
+                                 "Application versions (embedded in logfile):")
+            self.failUnless(lines[1].strip().startswith("foolscap:"), lines[1])
+            self.failUnless(lines[2].strip().startswith("twisted:"), lines[2])
+            mypid = os.getpid()
+            self.failUnlessEqual(lines[3].strip(), "PID: %s" % mypid, lines[3])
+            self.failUnlessEqual(lines[4].strip(), "")
+            self.failIf("[INCIDENT-TRIGGER]" in lines[5])
+            self.failIf("[INCIDENT-TRIGGER]" in lines[6])
+            self.failUnless(lines[7].strip().endswith(": boom [INCIDENT-TRIGGER]"))
         d.addCallback(_check)
         return d
 
@@ -1949,6 +2010,11 @@ class Web(unittest.TestCase):
         d.addCallback(_created)
         d.addCallback(lambda res: client.getPage(self.url))
         def _check_welcome(page):
+            mypid = os.getpid()
+            self.failUnless("PID %s" % mypid in page,
+                            "didn't see 'PID %s' in '%s'" % (mypid, page))
+            self.failUnless("Application Versions:" in page, page)
+            self.failUnless("foolscap: %s" % foolscap.__version__ in page, page)
             self.failUnless("4 events covering" in page)
             self.failUnless('href="summary/0-20">3 events</a> at level 20'
                             in page)
@@ -1974,6 +2040,15 @@ class Web(unittest.TestCase):
         d.addCallback(lambda res:
                       client.getPage(self.baseurl + "all-events?sort=time"))
         d.addCallback(_check_all_events)
+        d.addCallback(lambda res:
+                      client.getPage(self.baseurl + "all-events?sort=nested"))
+        d.addCallback(_check_all_events)
+        d.addCallback(lambda res:
+                      client.getPage(self.baseurl + "all-events?timestamps=local"))
+        d.addCallback(_check_all_events)
+        d.addCallback(lambda res:
+                      client.getPage(self.baseurl + "all-events?timestamps=utc"))
+        d.addCallback(_check_all_events)
         return d
 
 
